#include <os/Process.h>

#include <errno.h>
#include <sys/syscall.h>
#include <unistd.h>

namespace os
{
    Result<pid_t> Process::fork()
    {
        long rc = syscall(SYS_fork);
        return Result<pid_t>::from_syscall(rc);
    }

    Result<void> Process::exec(StringView path, Slice<String> args, bool search_in_path)
    {
        Vector<const char*> argv;
        for (const auto& arg : args) { TRY(argv.try_append(arg.chars())); }
        TRY(argv.try_append(nullptr));

        if (search_in_path) execvp(path.chars(), const_cast<char**>(argv.data()));
        else
            execv(path.chars(), const_cast<char**>(argv.data()));

        return err(errno);
    }
}
